@@ -13,6 +13,7 @@
 #include <QDBusConnection>
 #include <QPixmap>
 #include <QColor>
+#include <QFontMetrics>
 #include "k9call.h"
 
 #include <QtDebug>
@@ -23,6 +24,8 @@ static const char* KEYPAD_IDX_PROPERTY = "keypadidx";
 
 static const unsigned int CACHE_VERSION = 9;
 
+static const int MAX_NAME_SIZE = 30 /*pt*/;
+
 // The QtMobility framework doesn't handle unicode strings correctly
 static inline QString fixUTF8String(const QString& str) {
 	const char* temp = str.toLatin1().data();
@@ -200,15 +203,37 @@ void k9call::nextContact() {
 	if(m_ActiveSet.size() == 0) {
 		ui.name->setText("");
 		ui.name->setEnabled(false);
+		ui.pic->scene()->clear();
 		return;
 	}
 
 	QContactLocalId id = *m_ActiveSetItr;
     QContact contact = m_Manager.contact(id);
 	QContactName name = contact.detail(QContactName::DefinitionName);
-
-	// Code to correct the conversion
 	QString display = fixUTF8String(name.firstName()+" "+name.lastName());
+
+	// Figure out optimal font sizes
+	QFont font = ui.name->font();
+	QFontMetrics fontmetric(font);
+	const int nameWidth = ui.name->width()-10;
+	const int nameHeight = (ui.name->height()-10);
+	// Enlarge the font as necessary
+	while(fontmetric.width(display) < nameWidth &&
+	      fontmetric.height() < nameHeight &&
+	      font.pointSize() < MAX_NAME_SIZE) {
+		// Font is too small, make it bigger
+		font.setPointSize(font.pointSize()+1);
+		fontmetric = QFontMetrics(font);
+	}
+	// Shrink to fit
+	while(fontmetric.width(display) > nameWidth ||
+	      fontmetric.height() > nameHeight || 
+	      font.pointSize() == 1) {
+		// Font is too small, make it bigger
+		font.setPointSize(font.pointSize()-1);
+		fontmetric = QFontMetrics(font);
+	}
+	ui.name->setFont(font);
 	ui.name->setText(display);
 	ui.name->setEnabled(true);
 
@@ -217,8 +242,7 @@ void k9call::nextContact() {
 	QImage thumbnail = detail.thumbnail().convertToFormat(QImage::Format_RGB32);
 	if(thumbnail.isNull()) {
 		// No thumbnail so just clear the pic
-		QGraphicsScene *scene = ui.pic->scene();
-		scene->clear();
+		ui.pic->scene()->clear();
 	}
 	else {
 		// Scale the image to fit (upscale until Nokia pushes the fix thorugh, same bug as below)
